#pragma hdrstop

#include "OAdapterUtils.h"
#include "OUtils.h"
//#include <Forms.hpp>


#pragma package(smart_init)

//#pragma link "OUtils"

using namespace OAdapterUtils;
using namespace OUtils;

////////////////////////////////////////////////////////////////////////////////
//
// OAdapter::CFields::...
//
const Variant CFields::operator [](const AnsiString Index)
{
    try
    {
        const_iterator Field = find(Index);
        if (Field != end()) return Field->second;
        else
        {
            ErrorMessage(AnsiString("   \"" + Index + "\"  "));
            return varNull; //static_cast<Variant>(NULL);
        }
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  []  OAdapterUtils::CFields,   \"" + Index + "\"", E);
        return varNull;
    } catch (...)
    {
        CatchEllipsisMessage("   []  OAdapterUtils::CFields,   \"" + Index + "\"");
        return varNull;
    }
}

////////////////////////////////////////////////////////////////////////////////
bool CFields::Insert(const AnsiString Index, const Variant Value)
{
    try
    {
        const_iterator Field = find(Index);
        if (Field == end())
        {
            map<AnsiString, Variant>::operator [](Index) = Value;
            return true;
        }
        return false;
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  Insert  OAdapterUtils::CFields", E);
        return false;
    } catch (...)
    {
        CatchEllipsisMessage("   Insert  OAdapterUtils::CFields");
        return false;
    }
}

////////////////////////////////////////////////////////////////////////////////
bool CFields::Delete(const AnsiString Index)
{
    try
    {
        const_iterator Field = find(Index);
        if (Field != end())
        {
            erase(Index);
            return true;
        }
        return false;
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  Delete  OAdapterUtils::CFields", E);
        return false;
    } catch (...)
    {
        CatchEllipsisMessage("   Delete  OAdapterUtils::CFields");
        return false;
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// OAdapter::CResFields::...
//
/*
OAdapter::CFields* OAdapter::CResFields::GetFields()
{
    return second.get();
}
*/

////////////////////////////////////////////////////////////////////////////////
/*
bool OAdapter::CResFields::GetValid()
{
    return Executed; // && (bool)(Table->Count);
}
*/

////////////////////////////////////////////////////////////////////////////////
CResFields::CResFields():
       pair<bool, PFields>(false, PFields(new CFields()))
{
}

////////////////////////////////////////////////////////////////////////////////
CResFields::CResFields(bool Executed):
       pair<bool, PFields>(Executed, PFields(new CFields()))
{
}

////////////////////////////////////////////////////////////////////////////////
const Variant CResFields::operator [] (const AnsiString Index)
{
    try
    {
        if (!second.get()) return varNull;
        return (*(second.get()))[Index];
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  []  OAdapterUtils::CResFields", E);
        return varNull;
    } catch (...)
    {
        CatchEllipsisMessage("   []  OAdapterUtils::CResFields");
        return varNull;
    }
}

////////////////////////////////////////////////////////////////////////////////
bool CResFields::Insert(const AnsiString Index, const Variant Value)
{
    try
    {
        if (!second.get()) return false;
        return second.get()->Insert(Index, Value);
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  Insert  OAdapterUtils::CResFields", E);
        return false;
    } catch (...)
    {
        CatchEllipsisMessage("   Insert  OAdapterUtils::CResFields");
        return false;
    }
}

////////////////////////////////////////////////////////////////////////////////
bool CResFields::Delete(const AnsiString Index)
{
    try
    {
        if (!second.get()) return false;
        return second.get()->Delete(Index);
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  Delete  OAdapterUtils::CResFields", E);
        return false;
    } catch (...)
    {
        CatchEllipsisMessage("   Delete  OAdapterUtils::CResFields");
        return false;
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// OAdapter::CTable::...
//
CTable::CTable(): UniqueIndex(0)
{
}

////////////////////////////////////////////////////////////////////////////////
void CTable::Insert(CFields& const Fields)
{
    try
    {
        map<int, CFields>::operator[](UniqueIndex++) = Fields;
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  Insert  OAdapterUtils::CTable", E);
    } catch (...)
    {
        CatchEllipsisMessage("   Insert  OAdapterUtils::CTable");
    }
}
/*
////////////////////////////////////////////////////////////////////////////////
CFields& const CTable::Insert()
{
    CFields f;
    map<int, CFields>::operator[](UniqueIndex++) = f;
    return map<int, CFields>::operator[](UniqueIndex);
}
*/

////////////////////////////////////////////////////////////////////////////////
bool CTable::Delete(const int Index)
{
    try
    {
        const_iterator Record = find(Index);
        if (Record != end())
        {
            erase(Index);
            return true;
        }
        return false;
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  Delete  OAdapterUtils::CTable", E);
        return false;
    } catch (...)
    {
        CatchEllipsisMessage("   Delete  OAdapterUtils::CTable");
        return false;
    }
}

////////////////////////////////////////////////////////////////////////////////
CFields& CTable::operator[](int Index)
{
    try
    {
        iterator Record = find(Index);
        if (Record != end())
        {
            return Record->second;
        }
        throw Exception("   [" + IntToStr(Index) + "]  ");
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  []  OAdapterUtils::CTable", E);
        //return false;
    } catch (...)
    {
        CatchEllipsisMessage("   []  OAdapterUtils::CTable");
        //return false;
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// OAdapter::CResTable::...
//
/*
OAdapter::CTable* OAdapter::CResTable::GetTable()
{
    return second.get();
}
*/

////////////////////////////////////////////////////////////////////////////////
bool CResTable::GetValid()
{
    return Executed && (bool)Count;
}

////////////////////////////////////////////////////////////////////////////////
int CResTable::GetCount()
{
    try
    {
        if (!second.get()) return 0;
        return second.get()->Count;
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  GetCount  OAdapterUtils::CResTable", E);
        return false;
    } catch (...)
    {
        CatchEllipsisMessage("   GetCount  OAdapterUtils::CResTable");
        return false;
    }
}

////////////////////////////////////////////////////////////////////////////////
CResTable::CResTable():
       pair<bool, PTable>(false, PTable(new CTable()))
{
}

////////////////////////////////////////////////////////////////////////////////
CResTable::CResTable(bool Executed):
       pair<bool, PTable>(Executed, PTable(new CTable()))
{
}

////////////////////////////////////////////////////////////////////////////////
CFields& CResTable::operator[](int Index)
{
    try
    {
        if (!second.get()) throw Exception("   ");
        return (*(second.get()))[Index];
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  []  OAdapterUtils::CResTable", E);
    } catch (...)
    {
        CatchEllipsisMessage("   []  OAdapterUtils::CResTable");
    }
}

////////////////////////////////////////////////////////////////////////////////
void CResTable::Insert(CFields& const Fields)
{
    try
    {
        if (!second.get()) throw Exception("   ");
        second->Insert(Fields);
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  Insert  OAdapterUtils::CResTable", E);
    } catch (...)
    {
        CatchEllipsisMessage("   Insert  OAdapterUtils::CResTable");
    }
}

/*
////////////////////////////////////////////////////////////////////////////////
CFields& const CResTable::Insert()
{
    return second->Insert();
}
*/

////////////////////////////////////////////////////////////////////////////////
bool CResTable::Delete(const int Index)
{
    try
    {
        if (!second.get()) return false;
        return second->Delete(Index);
    } catch (const Exception &E)
    {
        CatchExceptionMessage("  Delete  OAdapterUtils::CResTable", E);
        return false;
    } catch (...)
    {
        CatchEllipsisMessage("   Delete  OAdapterUtils::CResTable");
        return false;
    }
}